import os
from pathlib import Path

import click

API_KEY_LENGTH = 20
HOME = str(Path.home())
CREDS_FOLDER = f"{HOME}/.seamless"
CREDS_FILE = f"{CREDS_FOLDER}/credentials"


def get_api_key():
    if not os.path.exists(CREDS_FILE):
        click.echo("Cannot find credentials. "
                   "Please go to app.seamlesscloud.io, copy your api-key and run "
                   "'smls init <your api key>'")
        exit(0)
    with open(CREDS_FILE, 'r') as creds:
        api_key = creds.read()
    if not is_api_key_valid(api_key):
        click.echo("Credentials file does not contain a valid API KEY "
                   "Please go to app.seamlesscloud.io, copy your api-key and run "
                   "'smls init <your api key>'")
        exit(0)
    return api_key


def set_api_key(api_key):
    Path(CREDS_FOLDER).mkdir(parents=True, exist_ok=True)
    with open(CREDS_FILE, 'w+') as f:
        f.write(api_key)


def is_api_key_valid(api_key):
    if len(api_key) != API_KEY_LENGTH:
        return False
    elif not api_key.isalnum():
        return False
    elif not api_key.islower():
        return False
    else:
        return True



import yfinance as yf

tickers = [
    yf.Ticker("AAPL"),
    # ................
    # ^^ add tickers of companies you want to follow here
    # you can find a ticker from a company name here https://www.marketwatch.com/tools/quotes/lookup.asp
]


# By default Seamless Cloud will execute the function `main` in the file `function.py`
# You can override this behaviour by using --entrypoint flag
def main():
    for ticker in tickers:
        print(f"Company: {ticker.info['longName']}")
        print(f"Stock Price: ${ticker.info['regularMarketPrice']}")
        # Let's look at 3 latest recommendations by stock analysts firms
        print("Latest 3 recommendations:")
        for date, row in ticker.recommendations.iloc[-3:].iterrows():
            print(f"{row['Firm']} recommendation dated {date.date()}: {row['To Grade']}")
        # ................
        # ^^ here you can send email or message in your favourite messenger if you want



import responses
from click.testing import CliRunner

from smls.constants import SEAMLESS_SERVICE_URL, SEAMLESS_SERVICE_RUN_ROUTE
from smls.seamless import run


@responses.activate
def test_run_command():
    pass
    # response_str = "Running your code seamlessly"
    # request_url = SEAMLESS_SERVICE_URL + SEAMLESS_SERVICE_RUN_ROUTE
    # responses.add(
    #     responses.POST, request_url, body=response_str)
    #
    # runner = CliRunner()
    # result = runner.invoke(run)
    #
    # # The right request was made
    # assert len(responses.calls) == 1
    # assert responses.calls[0].request.url == request_url
    # assert responses.calls[0].response.text == response_str
    #
    # # The cli command printed the response text
    # assert response_str in result.output



PACKAGE_NAME = "smls"
PACKAGE_ENTRY_POINT = "smls"
ARCHIVE_FOR_SENDING_NAME = "seamless_package.tar.gz"
ARCHIVE_SIZE_LIMIT = 10 * 1024 * 1024  # 10 MB
SEAMLESS_HOST = "https://app.seamlesscloud.io"
# SEAMLESS_HOST = "http://localhost:5000"
SEAMLESS_SERVICE_URL = f"{SEAMLESS_HOST}/api/v1"
SEAMLESS_SERVICE_RUN_ROUTE = "/run"
SEAMLESS_SERVICE_PUBLISH_ROUTE = "/publish"
SEAMLESS_SERVICE_JOBS_ROUTE = "/jobs"
EXCLUDE_FOLDERS_AND_FILES = ['.git', '__pycache__', '.pytest_cache']



import pathlib

from setuptools import setup, find_packages

from smls.constants import PACKAGE_NAME, PACKAGE_ENTRY_POINT

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

setup(
    name=PACKAGE_NAME,
    version_config={
        "version_format": "{tag}.dev{sha}",
        "starting_version": "0.0.1"
    },
    setup_requires=['better-setuptools-git-version'],
    description="Command Line Interface to work with the Seamless Cloud product",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/https://github.com/seamless-io/seamless-cli",
    author="Seamless Cloud",
    author_email="hello@seamlesscloud.io",
    license="ISC",
    classifiers=[
        "License :: OSI Approved :: ISC License (ISCL)",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
    packages=find_packages(),
    include_package_data=True,
    install_requires=["setuptools-git",
                      "click~=7.1.2",
                      "requests~=2.23.0",
                      "responses~=0.10.15"],
    entry_points="""
        [console_scripts]
        {entry_point}=smls.seamless:cli
    """.format(entry_point=PACKAGE_ENTRY_POINT),
)



from subprocess import run, PIPE, STDOUT

import pytest

collect_ignore = ["setup.py"]

from smls.constants import PACKAGE_NAME


def execute_terminal_command(command):
    result = run(command, stdout=PIPE, stderr=STDOUT)
    return result.stdout.decode("utf-8")


@pytest.fixture(autouse=True, scope="session")
def install_the_package():
    execute_terminal_command(["pip", "install", "--editable", "."])
    yield
    execute_terminal_command(["pip", "uninstall", "-y", PACKAGE_NAME])



from smls.constants import PACKAGE_ENTRY_POINT
from tests.integration.conftest import execute_terminal_command


def test_init_command():
    # output = execute_terminal_command([PACKAGE_ENTRY_POINT, "init"])
    # assert "Cannot find credentials" in output
    pass  # TODO need operational service in order to finish this integration test


def test_run_command_works():
    pass  # TODO need operational service in order to finish this integration test


def test_publish_command_works():
    pass  # TODO need operational service in order to finish this integration test



import os
import tarfile
from distutils.dir_util import copy_tree

import click
import requests

from smls.api_key import get_api_key, set_api_key, is_api_key_valid
from smls.constants import ARCHIVE_FOR_SENDING_NAME, SEAMLESS_SERVICE_URL, ARCHIVE_SIZE_LIMIT, \
    EXCLUDE_FOLDERS_AND_FILES, SEAMLESS_SERVICE_RUN_ROUTE, SEAMLESS_SERVICE_PUBLISH_ROUTE, SEAMLESS_HOST, \
    SEAMLESS_SERVICE_JOBS_ROUTE


def _package_project(folder_to_archive):
    def filtr(file):
        for name in EXCLUDE_FOLDERS_AND_FILES:
            if file.name.startswith(f"./{name}"):
                return None
        return file

    tar = tarfile.open(ARCHIVE_FOR_SENDING_NAME, "w:gz")
    tar.add(folder_to_archive, filter=filtr)
    tar.close()
    # TODO we should have this limit only on server side and return meaningful error
    # TODO it should be doable with the production WSGI server and catching 413 error
    if os.stat(ARCHIVE_FOR_SENDING_NAME).st_size > ARCHIVE_SIZE_LIMIT:
        print(f"Your project is too big."
              f" After we compressed the folder {folder_to_archive} the resulting file is more than 10 MB."
              f"In order to have a really seamless experience we recommend to remove non-essential files."
              f"If you need to increase the limit - please shoot us an email at hello@seamlesscloud.io")
        exit(1)
    return ARCHIVE_FOR_SENDING_NAME


@click.group()
# This is just an entry point for the CLI, all other commands are registered in this group
def cli():
    pass


@cli.command()
@click.option('-e', '--entrypoint', default='function.main', help='Dot-separated path to the function')
@click.option('-r', '--requirements', default='requirements.txt', help='Path to the file with requirements')
def run(entrypoint, requirements):
    api_key = get_api_key()
    package_name = None
    try:
        package_name = _package_project(folder_to_archive='.')
        resp = requests.post(SEAMLESS_SERVICE_URL + SEAMLESS_SERVICE_RUN_ROUTE,
                             params={'entrypoint': entrypoint, 'requirements': requirements},
                             headers={'Authorization': api_key},
                             files={'seamless_project': open(package_name, 'rb')},
                             stream=True)
        try:
            resp.raise_for_status()
        except requests.HTTPError:
            click.echo(resp.text)
            exit(1)
        for line in resp.iter_lines(decode_unicode=False, chunk_size=1):
            click.echo(line)
    finally:
        try:
            if package_name:
                os.remove(package_name)
        except OSError:
            pass


@cli.command()
@click.option(
    "-n",
    "--name",
    help="name of the job you want to publish",
    required=True
)
@click.option(
    "-s",
    "--schedule",
    help="cron expression that identifies the schedule your code runs on",
)
@click.option('-e', '--entrypoint', default='function.main', help='Dot-separated path to the function')
@click.option('-r', '--requirements', default='requirements.txt', help='Path to the file with requirements')
def publish(name, schedule, entrypoint, requirements):
    api_key = get_api_key()
    package_name = None
    try:
        package_name = _package_project(folder_to_archive='.')
        params = {'name': name, 'entrypoint': entrypoint, 'requirements': requirements}
        if schedule:
            params.update({'schedule': schedule})
        resp = requests.put(SEAMLESS_SERVICE_URL + SEAMLESS_SERVICE_PUBLISH_ROUTE,
                            params=params,
                            headers={'Authorization': api_key},
                            files={'seamless_project': open(package_name, 'rb')})
        try:
            resp.raise_for_status()
        except requests.HTTPError:
            click.echo(resp.text)
            exit(1)
        data = resp.json()
        link_to_job = f"{SEAMLESS_HOST}/jobs/{data['job_id']}"
        if not data['existing_job']:
            click.echo(f"Congratulations, you have created a new job. "
                       f"You can find it at {link_to_job}")
        else:
            click.echo(f"Congratulations, you have updated your '{name}' job. "
                       f"You can find it at {link_to_job}")
    finally:
        try:
            if package_name:
                os.remove(package_name)
        except OSError:
            pass


@cli.command()
@click.option(
    "--name",
    help="name of the job you want to remove",
    required=True
)
def remove(name):
    api_key = get_api_key()
    resp = requests.delete(SEAMLESS_SERVICE_URL + SEAMLESS_SERVICE_JOBS_ROUTE + f'/{name}',
                           headers={'Authorization': api_key})
    try:
        resp.raise_for_status()
    except requests.HTTPError:
        click.echo(resp.text)
        exit(1)

    click.echo(f"Job '{name}' was removed.")


@cli.command()
@click.argument('api_key')
def auth(api_key):
    if not is_api_key_valid(api_key):
        click.echo("The API KEY provided is not valid. "
                   "API key should be a sting exactly 20 characters long.")
        exit(0)
    set_api_key(api_key)

    click.echo("Welcome to the Seamless Cloud community!")


@cli.command()
def example():
    example_job_folder_name = 'stock_monitoring_job'
    source = os.path.join(os.path.dirname(os.path.realpath(__file__)),
                          'example_job')
    destination = os.path.join(os.getcwd(), example_job_folder_name)
    copy_tree(source, destination)

    click.echo(f"Example job was created! Now let's run and publish it. In the terminal do the following:")
    click.echo(f"'cd {example_job_folder_name}' - to go into the folder with the example job")
    click.echo(f"'smls run' - to execute the job on the Seamless Cloud server")
    click.echo(f"'smls publish --name \"Stock Price Monitoring\" --schedule \"0 0 * * *\"' - "
               f"to run the job every day at 00:00 UTC")
